"""ॐ
    Version 0.0.1: 2D MEDCNN without attentions

    MEDCNN: Multiresolution Encoder-Decoder Convolutional Neural Network
    Copyright (C) 2025 Kishore Kumar Tarafdar
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
    """



# import tensorflow as tf
# import keras
# import numpy as np

# from TFDWT.dbFBimpulseResponse import FBimpulseResponses
# from TFDWT.GETDWTFiltersOrtho import GETDWTFiltersOrtho
# from TFDWT.get_A_matrix_dwt_analysisFB_unit import get_A_matrix_dwt_analysisFB_unit

# # from TFDWT3D.DWTIDWT1Dv1 import DWT1D, IDWT1D
# # from TFDWT3D.DWTIDWT2Dv1 import DWT2D, IDWT2D





__version__="0.0.1"